@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright (c) 2007 - 2012, Pascal Volk
+# Copyright (c) 2007 - 2014, Pascal Volk
 # See COPYING for distribution information.
 """
    VirtualMailManager.handler
@@ -56,6 +56,7 @@ OTHER_TYPES = {
     TYPE_RELOCATED: (_(u'a relocated user'), RELOCATED_EXISTS),
 }
 
+
 class Handler(object):
     """Wrapper class to simplify the access on all the stuff from
     VirtualMailManager"""
@@ -521,7 +522,8 @@ class Handler(object):
                            INVALID_ARGUMENT)
         dom = self._get_domain(domainname)
         dominfo = dom.get_info()
-        if dominfo['domain name'].startswith('xn--'):
+        if dominfo['domain name'].startswith('xn--') or \
+           dominfo['domain name'].count('.xn--'):
             dominfo['domain name'] += ' (%s)' % \
                                       dominfo['domain name'].decode('idna')
         if details is None:
@@ -621,7 +623,8 @@ class Handler(object):
                     checkp = lpattern
                 if len(checkp) > 0 and re.search(RE_LOCALPART, checkp):
                     raise VMMError(_(u"The pattern '%s' contains invalid "
-                                     u"characters.") % pattern, LOCALPART_INVALID)
+                                     u"characters.") % pattern,
+                                   LOCALPART_INVALID)
             else:
                 # else just match on domains
                 # (or should that be local part, I don't know…)
@@ -658,8 +661,8 @@ class Handler(object):
         alias = self._get_alias(aliasaddress)
         if not alias:
             self._is_other_address(alias.address, TYPE_ALIAS)
-        destinations = [DestinationEmailAddress(addr, self._dbh) \
-                for addr in targetaddresses]
+        destinations = [DestinationEmailAddress(addr, self._dbh)
+                        for addr in targetaddresses]
         warnings = []
         destinations = alias.add_destinations(destinations, warnings)
         if warnings:
@@ -709,23 +712,34 @@ The account has been successfully deleted from the database.
             raise VMMError(_(u"The alias '%s' does not exist.") %
                            alias.address, NO_SUCH_ALIAS)
 
-    def alias_delete(self, aliasaddress, targetaddress=None):
+    def alias_delete(self, aliasaddress, targetaddresses=None):
         """Deletes the `Alias` *aliasaddress* with all its destinations from
-        the database. If *targetaddress* is not ``None``, only this
-        destination will be removed from the alias."""
+        the database. If *targetaddresses* is not ``None``, only the given
+        destinations will be removed from the alias."""
         alias = self._get_alias(aliasaddress)
-        if targetaddress is None:
+        error = None
+        if targetaddresses is None:
             alias.delete()
         else:
-            alias.del_destination(DestinationEmailAddress(targetaddress,
-                                                          self._dbh))
+            destinations = [DestinationEmailAddress(addr, self._dbh)
+                            for addr in targetaddresses]
+            warnings = []
+            try:
+                alias.del_destinations(destinations, warnings)
+            except VMMError, err:
+                error = err
+            if warnings:
+                self._warnings.append(_('Ignored destination addresses:'))
+                self._warnings.extend(('  * %s' % w for w in warnings))
+            if error:
+                raise error
 
     def catchall_add(self, domain, *targetaddresses):
         """Creates a new `CatchallAlias` entry for the given *domain* with
         the given *targetaddresses*."""
         catchall = self._get_catchall(domain)
-        destinations = [DestinationEmailAddress(addr, self._dbh) \
-                for addr in targetaddresses]
+        destinations = [DestinationEmailAddress(addr, self._dbh)
+                        for addr in targetaddresses]
         warnings = []
         destinations = catchall.add_destinations(destinations, warnings)
         if warnings:
@@ -742,16 +756,27 @@ The account has been successfully deleted from the database.
         instances) for the `CatchallAlias` with the given *domain*."""
         return self._get_catchall(domain).get_destinations()
 
-    def catchall_delete(self, domain, targetaddress=None):
+    def catchall_delete(self, domain, targetaddresses=None):
         """Deletes the `CatchallAlias` for domain *domain* with all its
-        destinations from the database. If *targetaddress* is not ``None``,
-        only this destination will be removed from the alias."""
+        destinations from the database.  If *targetaddresses* is not
+        ``None``,  only those destinations will be removed from the alias."""
         catchall = self._get_catchall(domain)
-        if targetaddress is None:
+        error = None
+        if targetaddresses is None:
             catchall.delete()
         else:
-            catchall.del_destination(DestinationEmailAddress(targetaddress,
-                                                             self._dbh))
+            destinations = [DestinationEmailAddress(addr, self._dbh)
+                            for addr in targetaddresses]
+            warnings = []
+            try:
+                catchall.del_destinations(destinations, warnings)
+            except VMMError, err:
+                error = err
+            if warnings:
+                self._warnings.append(_('Ignored destination addresses:'))
+                self._warnings.extend(('  * %s' % w for w in warnings))
+            if error:
+                raise error
 
     def user_info(self, emailaddress, details=None):
         """Wrapper around Account.get_info(...)"""
@@ -813,7 +838,7 @@ The account has been successfully deleted from the database.
         if not acc:
             raise VMMError(_(u"The account '%s' does not exist.") %
                         acc.address, NO_SUCH_ACCOUNT)
-        if bytes_ == 'default':
+        if bytes_ == 'domain':
             quotalimit = None
         else:
             if not all(isinstance(i, (int, long)) for i in (bytes_, messages)):
@@ -832,7 +857,7 @@ The account has been successfully deleted from the database.
         if not acc:
             raise VMMError(_(u"The account '%s' does not exist.") %
                            acc.address, NO_SUCH_ACCOUNT)
-        if transport == 'default':
+        if transport == 'domain':
             transport = None
         else:
             transport = Transport(self._dbh, transport=transport)
@@ -844,7 +869,7 @@ The account has been successfully deleted from the database.
         if not acc:
             raise VMMError(_(u"The account '%s' does not exist.") %
                         acc.address, NO_SUCH_ACCOUNT)
-        if len(services) == 1 and services[0] == 'default':
+        if len(services) == 1 and services[0] == 'domain':
             serviceset = None
         else:
             kwargs = dict.fromkeys(SERVICES, False)
